﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Xml;
using System.Windows.Forms;

using FarPoint.Win.Spread;

namespace FP_CopyPaste
{
    public partial class Form1 : Form
    {
        private string m_CurrentPath;
        private string m_CurrentFileName;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.Text = "Farpoint Copy Paste Test";
            //this.Text = "Farpoint Celltype Interaction Test";
        }

        private void btnImportXML_Click(object sender, EventArgs e)
        {
            // Load a default file name and path but give the user the option to change.
            string fileName = m_CurrentFileName;
            string fullPath = OpenXmlFile(m_CurrentPath, fileName);

            if (fullPath == null || fullPath == string.Empty)
            {
                MessageBox.Show("Open File not selected; operation aborted.");
                return;
            }

            // Save path for later re-use
            // And the name for display
            int k = fullPath.LastIndexOf("\\");
            if (k >= 0)
            {
                m_CurrentPath = fullPath.Substring(0, k);
                fileName = fullPath.Substring(k + 1);
                m_CurrentFileName = fileName;
            }

            try
            {
                // Check if this is a valid FpSpread XML file
                if (!fpSpread1.IsSpreadXmlFile(fullPath))
                {
                   MessageBox.Show ( "Opened File Error: [" + fullPath + "] is not a valid FpSpread XML File.");
                    return;
                }

                // Directly load xml file into the spread
                fpSpread1.Open(fullPath);

            }
            catch (Exception ex)
            {
                MessageBox.Show ("Opened XML File Error: [" + fullPath + "] " + ex.Message);
                return;
            }

        }

        private void btnCopySelected_Click(object sender, EventArgs e)
        {
            // Copy the currently selected items to the clipboard
            fpSpread1.ActiveSheet.ClipboardCopy(); 
            
        }

        private void btnCutSelected_Click(object sender, EventArgs e)
        {
            // Cut the currently selected items to the clipboard
            fpSpread1.ActiveSheet.ClipboardCut(); 
        }

        private void btnPasteSelected_Click(object sender, EventArgs e)
        {
            // Cut the currently selected items to the clipboard
            fpSpread1.ActiveSheet.ClipboardPaste(); 
        }

        private void btnExportXML_Click(object sender, EventArgs e)
        {
            string fileName = m_CurrentFileName;
            string fullPath = SaveToFile(m_CurrentPath, fileName);

            if (fullPath == null || fullPath == string.Empty)
            {
                MessageBox.Show("Export XML File not selected; operation aborted.");
                return;
            }

            try
            {
                // Extract the Spreads XML for the sheet
                XmlDocument xmlDoc = GetFpSpreadXmlDocument(fpSpread1);

                // Save path for later use
                int k = fullPath.LastIndexOf("\\");
                if (k >= 0)
                {
                    m_CurrentPath = fullPath.Substring(0, k);
                    fileName = fullPath.Substring(k + 1);
                }

                xmlDoc.Save(fullPath);

            }
            catch (Exception ex)
            {
                MessageBox.Show("Save File Error: [" + fullPath + "] " + ex.Message);
                return;
            }

            MessageBox.Show("Saved File to " + fullPath);
        }

        /// <summary>
        /// Open a dialog box for XML Files
        /// </summary>
        /// <param name="initialDirectory"></param>
        /// <param name="initialFileName"></param>
        /// <returns>Name and path of the File</returns>
        private string OpenXmlFile(string initialDirectory, string initialFileName)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*";
            dialog.InitialDirectory = initialDirectory;
            dialog.FileName = initialFileName;
            dialog.Title = "Select a Farpoint XML file";
            return (dialog.ShowDialog() == DialogResult.OK)
               ? dialog.FileName : null;
        }

        /// <summary>
        /// Open a dialog box to select a file to export to
        /// </summary>
        /// <param name="initialDirectory"></param>
        /// <param name="initialFileName"></param>
        /// <returns></returns>
        private string SaveToFile(string initialDirectory, string initialFileName)
        {
            string functionReturnValue = null;
            SaveFileDialog dialog = new SaveFileDialog();
            functionReturnValue = Application.StartupPath + "\\test.xml";

            dialog.DefaultExt = "xml";
            dialog.CheckFileExists = false;
            dialog.InitialDirectory = initialDirectory;
            dialog.FileName = initialFileName;
            dialog.Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*";
            dialog.Title = "Export to a Farpoint XML file";
            dialog.ShowDialog();
            if (dialog.FileName != null)
            {
                functionReturnValue = dialog.FileName;
            }
            return functionReturnValue;
        }

        /// <summary>
        /// Get Farpoint Spread XML Document
        /// Extract the spreadsheet on the form into a memory stream which is then loaded into a XML document
        /// </summary>
        /// <param name="fpSpread"></param>
        /// <returns>returns XML document of structure and data in the Spread</returns>
        private XmlDocument GetFpSpreadXmlDocument(FarPoint.Win.Spread.FpSpread fpSpread)
        {
            // Serialize the Spread to Memory
            System.IO.Stream xmlStreamMem;
            xmlStreamMem = new System.IO.MemoryStream();
            FarPoint.Win.Spread.Model.SpreadSerializer.SaveXml(fpSpread, xmlStreamMem, false);

            // Move the pointer back to the start of the stream.
            // This prevents the error "Root element is missing" from occurring on the xml document load.
            xmlStreamMem.Seek(0, System.IO.SeekOrigin.Begin);

            // Load the Stream into a XML Document 
            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.Load(xmlStreamMem);
            xmlStreamMem.Close();

            // return the XML Document
            return (xmlDoc);

        }

        private void fpSpread1_EditError(object sender, FarPoint.Win.Spread.EditErrorEventArgs e)
        {
            // Invalid Cell Formula entered.  Farpoint will eat this with no message by default.
            if (e.EditError == EditError.InvalidCellFormula)
            {
                // Set a #VALUE! error instead of displaying nothing on a bad formula.
                Cell currentCell = fpSpread1.ActiveSheet.Cells[e.Row, e.Column];
                currentCell.Formula = null;
                currentCell.Value = FarPoint.CalcEngine.CalcError.Number;
            }

        }

        private void fpSpread1_UserFormulaEntered(object sender, UserFormulaEnteredEventArgs e)
        {

        }

        private void fpSpread1_Change(object sender, ChangeEventArgs e)
        {
            if (!fpSpread1.AllowUserFormulas)
            {
                MessageBox.Show("fpSpread1.AllowUserFormulas == false");
                //fpSpread1.AllowUserFormulas = true;
            }

        }

    }
}
